

/*
					CASE STUDY #1
Incidence of Phomopsis leaf blight of strawberry in response
to fungicide treatment

trt		Treatment (i=1,...,8, including control [8]).
blk		Block (j=1,...,4), assumed that block is random, ~N(0,sigma2_b)
n		Number of observed leaflets in experimental unit (n=75)
y		Number of diseased leaflets in experimental unit 
		(plot: trt x blk combination)
		y | blk_j ~ Binomial(p_ij,n), the conditional distribution
prop	Proportion diseased (y/n)
angular	Angular transformation (arcsine-square-root of proportion)

Data set previously used in Madden et al. (PLANT DISEASE 86:316-325), 
based on data collected by Nita, Madden, and Ellis. 
In the original data collection, there were five samples of 15 
leaflets each in each plot (experimental unit). Here, we pooled 
these into one sample of n=75 leaflets. In a later case
study (#3), we show how to analyze the full "sub-sampled" data set. 

Following code is provided for educational purposes with no 
warranties.  

						-- Larry Madden and Alissa Kriss (2016)
*/


data pooled;
input trt blk y n;
prop = y/n;					*<--proportion diseased;
angular =arsin(sqrt(prop));	*<--arcsine-square-root transformation (angular);
datalines;
1 1 13 75
2 1 12 75
3 1 19 75
4 1 22 75
5 1 3 75
6 1 11 75
7 1 0 75
8 1 48 75
1 2 32 75
2 2 21 75
3 2 5 75
4 2 14 75
5 2 1 75
6 2 7 75
7 2 2 75
8 2 26 75
1 3 18 75
2 3 26 75
3 3 14 75
4 3 8 75
5 3 2 75
6 3 2 75
7 3 0 75
8 3 37 75
1 4 26 75
2 4 25 75
3 4 6 75
4 4 1 75
5 4 4 75
6 4 5 75
7 4 7 75
8 4 28 75
;
run;

title 'Case study #1: Phomopsis leaf blight incidence';
proc print data=pooled;
run;


*---First do tradational analyses based on proportions or angular
	transformation of proportions. That is, assume a linear mixed
	model (LMM) with normally distributed y.;

ods graphics on / width=5in;
proc sgplot data=pooled;
scatter x=trt y=prop / markerattrs=(color=blue symbol=circlefilled size=7pt);
xaxis  label="Treatment" type=discrete;
yaxis label="Disease incidence";
run;

proc sgplot data=pooled;
vbox prop / category=trt ;
xaxis  label="Treatment" type=discrete;
yaxis label="Disease incidence";
run;


*---First analyze data as if proportion had a normal conditional distribution.
	This gives unbiased estimates of the mean proportion diseased across all
	random effect (block) levels, which is a marginal distribution result.
	All SEs are wrong because the proporition variance must be a function of
	p(1-p).;

proc glimmix data=pooled ;
	title2 'LMM, y:proportion, fixed trt, random blk';
	class trt blk;
	model prop = trt ;
	random  blk;
	lsmeans trt / lines cl;	*<--"cl" for conf. int., "lines" for multiple comparisons;
run;

*---Now analyze the angular transformation. That is, assume that 
	the arcsine-square-root transformed variable is normally distributed
	with a constant variance. This (mostly) solves the unequal variance
	problem when analyzing the proportions directly. However, the estimated
	mean is a marginal distribuion result. The back-transformation of the
	angular mean is NOT the mean proportion across all blocks and is 
	NOT the mean conditional proportion for an experimental unit. It is
	something in between.;

proc glimmix data=pooled ;
	title2 'LMM, y:angular transformation, fixed trt, random blk';
	class trt blk;
	model angular = trt ;
	random  blk;
	lsmeans trt / lines cl;
run;


/*
Now perform analysis using a Generalized Linear Mixed Model (GLMM).
In GLIMMIX, y and n and the distribution of y are given in the MODEL 
statement. The link function is also given in the MODEL 
statement (default is LOGIT for binomial, but it is written out below
for clarity). Also, when one specifies y/n on the left-hand side of
the = sign, GLIMMIX assumes that y has a conditional binomial 
distribution. As with logit link option, we explicitly specify the
distribution for completeness and clarity.

Random effects (and any special structure to
the random effects) are given in RANDOM statements 
(there can be any number of RANDOM statements). 

LSMEANS statement is used to obtain estimated Least Squares
Means on the link scale (logit of p^). The ILINK option (shown later)
requests the inverse link of the least square means (returning to 
the disease incidence [p^] scale). "cl" option requests
confidence intervals (95%). "diff" option requests differences
of least squares means (link scale). "lines" option requests 
mean separation tables with letters for those means not 
signficantly different.

There are many options to the PLOT=MEAN option, which is used to
visualize the LSMEANS (which are logits of p^_ij). The default
is to plot on the link scale. Here, that means plot the logits. 
Below we show the additional ilink option in the plot option to
graph the estimated p^_ij values (the inverse links). When you 
look at the graph with default options, it is not obvious that
one is seeing logits (or whatever link is used) -- be careful.
*/ 

proc glimmix data=pooled ;
	title2 'Conditional binomial distribution,  random blk';
	class trt blk;
	model y/n = trt /  dist=bin link=logit;
	random  blk ;
	lsmeans trt / cl plot=mean(CL) ;
run;


*---Get differences of least squares means (logits), and mean
	separations.;
proc glimmix data=pooled ;
	title2 'Conditional binomial distribution,  random blk';
	title3 'Differences of LSMEANS, and mean separation (letters)';
	class trt blk;
	model y/n = trt /  dist=bin link=logit;
	random  blk ;
	lsmeans trt / diff lines cl plot=mean(CL) ;
run;

*---Get differences of least squares means (logits), and also
	the inverse links (estimated probability of disease per 
	treatment), the scale of disease incidence as a proportion.
	Note that tabular output gives logits and estimated probabilities 
	(far right side). Mean plot is also on probability scale if ilink
	option is given in plot option. However,
	differences of means are _only_ for logits, not for 
	probabilities. This is becauses a difference of logits cannot
	be tranformed into a sensical statistical measure.;

proc glimmix data=pooled ;
	title2 'Conditional binomial distribution,  random blk';
	title3 'Differences of LSMEANS and Inverse Link of LSMEANS';
	class trt blk;
	model y/n = trt /  dist=bin link=logit;
	random  blk ;
	lsmeans trt / diff lines ilink cl plot=mean(ilink CL) ;
run;

*^--All of above runs are misleading (not correct!), because there is
	another source of variation that is not accounted for by blindly
	using the normal-distribution linear predictor for binomial data.
	Below we show proper ways to include the necessary terms in the
	linear predictor. First see the Powerpoint slides for explanations.;



*************************************************************************;
*********** Accounting for experimental-unit random effects *************;
*************************************************************************;

/*  Conditional model approach (using true likelihoods and GLMM  */

*---Now add a block*treatment random effect (blk*trt here) to
	account for unique random effects of each experimental unit (plot).
	This is a way to account for variability that is larger than one
	would achieve with a binomial conditional distribution.;
*---Because the observed leaflets are clustered in the plot, they are
	correlated, which translates into higher variance(s) than one 
	obtains with a binomial distribution [np(1-p)].; 

proc glimmix data=pooled ;	*<--this is one of the correct model fits;
	title2 'Conditional binomial distribution,  random BLK BLK*TRT';
	class trt blk;
	model y/n = trt / dist=bin link=logit;
	random  blk blk*trt;	*<--see the addition of blk*trt term;
	lsmeans trt / diff lines ilink cl plot=mean(CL) ;
run;
 

proc glimmix data=pooled ;  *<--this is one of the correct model fits;
	title2 'Conditional binomial distribution,  random BLK BLK*TRT';
	title3 'add the IVLINK to the plot and table';
	class trt blk;
	model y/n = trt / dist=bin link=logit;
	random  blk blk*trt;	*<--see the addition of blk*trt term;
	lsmeans trt / diff ilink lines  cl plot=mean(CL ilink) ;
run;

*^--Above, see that the standard errors of the means and mean 
	differences are all larger than without blk*trt. F statistic
	is smaller (bit less evidence of significant treatment effect,
	although still highly significant here). Confidence intervals
	are wider. There is a fairly large blk*trt variance -- thus,
	one should keep this term in the model!;
*^--Also note that the block variance estimate is now 0 (previously,
	the block term was 'trying' to represent some of the blk*trt
	effect). Once blk*trt was included, block main effect because
	unimportant in this data set. Each data set is different.;

*---One could take out the block main effect because its variance
	is 0. With the default model fitting method (here), removal of the
	term is not needed (one gets the same results with or without the
	blk main effect (with 0 variance)). See below for very important 
	comments if other model fitting methods (not default) are used!;

*---Now for an equivalent way to fit the same model, using the explicit
	SUBJECT= syntax (each block is a subject here). This can be more 
	computationally efficient for big data sets and complex models. 
	It can also be more intuitive (to some investigators), and it can lead
	to other model formulations in a fairly seamless manner.
	Depending on time, we may skip the alternative formulations.;

proc glimmix data=pooled ;
	title2 'Conditional binomial distribution,  random BLK BLK*TRT';
	title3 'Alternative way to write same model, using SUBJECT= syntax';
	class trt blk;
	model y/n = trt /  dist=bin link=logit;
	random  int trt / sub=blk; 	*---Same as "random blk blk*trt";
	lsmeans trt / diff lines ilink cl plot=mean(CL ilink) ;
run;
*^--"int" is intercept (think of this as a column of 1s [ones]).
		The concept is that one multiplies the subject variable by 
		every term before the "/";



*---One _can_ allow for a negative variance by using the NOBOUND
	option. The only requirement is that the sum of all variances	
	must be a positive number. There are some statistical advantages
	to negative variance components in terms of Type I error rates,
	and so on. Conceptually, it may be challenging to explain a 
	negative variance in the context of a conditional GLMM, since
	a random variable (including a random block effect) must have a
	non-negative variance. Demonstrate here, but we do not pursue
	further in this workshop.;
proc glimmix data=pooled NOBOUND;
	title2 'Conditional binomial distribution, NOBOUND, random BLK BLK*TRT';
	title3 'Differences of LSMEANS and Inverse Link of LSMEANS';
	class trt blk;
	model y/n = trt / dist=bin link=logit;
	random  blk blk*trt;
	lsmeans trt / diff lines ilink cl plot=mean(CL) ;
run;



/*** Quasi-likelihood approach (no unit-level random effect term) ***/


/*
		ALTERNATIVE (A HYBRID APPROACH USING QUASI-LIKELIHOOD):
There is another (DIFFERENT) way to deal with the extra variability 
that results from having all the n leaflets clustered within a plot.
This is a so-called quasi-likelihood approach. The idea is to 
consider the conditional variance of Y for a plot as:
	phi*n*p(1-p),
rather than
	n*p*(1-p).
Here, phi is a scale parameter that inflates the conditional binomial variance 
(if phi > 1). One is no longer doing likelihood-based analysis because 
the conditional statistical distribution is no longer defined (or definable). 
That is, one uses dist=bin in the procedure for the conditional
"distribution", but the binomial distribution does not (_cannot_) have
a form such as phi*n*p*(1-p). It is possible that _no_ definable
distribution has this form for binary data (i.e., with inflated variance).
In this case, there would be no plausible explanation for how the data 
were generated stochastically. Quasi-likelihood only really needs the
means and variances, and is a workaround for the inflated variance
problem. It can give satisfactory results for standard errors, F
statistics, and so on. It is, however, removing the model as a 
fully adequate mechanism for data generation.

This may be considered a hybrid approach.

This approach is also described in the SAS User's Guide for GLIMMIX (Example 1).
It is not given in Stroup Agronomy Journal article, but is in his textbook.
*/

proc glimmix data=pooled ;	*<--a correct approach to account for higher variances;
	title2 'Hybrid approach:  quasi-binomial, overdispersion, random blk';
	title3 'Quasi-likelihood (residual scale parameter)';
	class trt blk;
	model y/n = trt /  dist=bin link=logit;
	random  blk ;			*<--(we take out blk*trt, but put in a residual);
	random _residual_ ;		*<--special keyword to get a phi scale parameter estimate;
	lsmeans trt / diff lines ilink cl plot=mean(CL) ;
run;

*^--Compare above to model with just blk variance and to model with 
	random blk and blk*trt .
	Similar results compared with blk*trt true random effect in conditional 
	model (in this case). SEs are larger and F statistic is smaller than 
	when there is just blk variance (the latter being in incorrect model). ;
*^--VERY IMPORTANT: with this quasi-likelihood approach, the actual	
	parameter estimates are identical to the model with just
		random blk .
	Only the measures of variability (SEs) are changed (inflated). Thus, 
	this adjustment may not be fully adequate, because if blk*trt matters,
	it should affect the parameter estimates _and_ the variances.;


*---Following is WRONG (on purpose). Shows that one need to be careful 
	about so-called dealing with Overparameterization (too many terms). ;
proc glimmix data=pooled ;
	title2 'Deliberately WRONG:  quasi-binomial, overdispersion, AND random blk & blk*trt';
	title3 'Quasi-likelihood (residual scale parameter)';
	class trt blk;
	model y/n = trt /  dist=bin link=logit;
	random  blk blk*trt;	*<--(we left in blk*trt, and put in a residual: WRONG!);
	random _residual_ ;		*<--special keyword to get a scale parameter estimate;
	lsmeans trt / diff lines ilink cl plot=mean(CL) ;
run;

*^--The _residual_ and the blk*trt are competing to account for the unique plot
	random effect. Only one of these two can be used. You do not easily see
	anything wrong in the output -- one must know what model is being used;

*---One can fit the quasi-likelihood model using PQLglmm {MASS} in R.
	However: be very careful. With this function in R, one ALWAYS gets the estimated
	phi parameter (similar to always having the _residual_ GLIMMIX statement).
	With this R package/function, one could not have the unit-level variation
	term (blk*trt) -- if one included it, one would get meaningless results (as
	done in the previous wrong GLIMMIX run).;
	

/********************************************************************************
							General guidance:
Fit the GLMM by first borrowing from the LMM (normal conditional distribution)
model formulation (that is, identify all the sources of random variation, such as 
block).

Either add a unit-level random effects term (blk*trt here) OR a _residual_
scale (phi) parameter, NOT both. If using Laplace/quadrature, only the former
is possible. 

If using Laplace/quadrature, remove any random effects with 0 variance before
proceeding with other stuff.
*********************************************************************************/




******************************************************************************;
****************  Alternative model-fitting approach  ************************;
******************************************************************************;


/*
		MODEL FITTING BASED ON DIRECTLY APPROXIMATING THE LIKELIHOOD
In the above runs, the GLMM was fitted using pseudo-likelihood (linearization)
(method=rspl, the default). This is a doubly iterative method, where 
a pseudo-variable (and associated weight) is determined based
on the conditional distribution and random effects, and treated as a normally 
distributed variable in the inner iteration. With each convergence of the inner
iteration step, a new pseudo-variable (and weight) is created based
on the conditional distribution, and the inner iteration step is
repeated. Final convergence occurs when the inner and outer iteration
steps converge. In summary, the model fitting is done by approximating
the model; thus, the likelihood calculated (and displayed) is based
on the pseudo-variable. This makes is hard to determine goodness of
fit and also hard to perform inference on the random-effect terms 
(such as the variances). But, the pseudo-likelihood method is 
very flexible, and generally works for the most diverse collection of problems. 

There are alternatives that involve approximating the likelihood 
directly, on the scale of the data, and does not involve approximating
the model. This is useful for goodness of fit and for getting
the most realistic confidence intervals for variances, etc. The
most general approach is known as QUADRATURE, and a special case 
(that is much faster, and works for larger data sets) is known 
as the LAPLACE approximation method.

The Laplace/quadrature methods can only fit models to true conditional
distributions. Thus, one CANNOT use for quasi-likelihood. In other
words, one cannot use the _residual_ option.;

*/

proc glimmix data=pooled method=LAPLACE ;
	title2 'Conditional binomial distribution,  random BLK BLK*TRT';
	title3 'LAPLACE estimation method (approximate the likelihood)';
	class trt blk;
	model y/n = trt / dist=bin link=logit;
	random  blk blk*trt;
	lsmeans trt / diff lines ilink cl plot=mean(CL) ;
run;
*^--Note that things blow up badly because of the 0 for block variance (here).
	One gets absurdly large test statistics and absurdly small SEs. 
			---->MEANINGLESS RESULTS FOR EXAMPLE<----
	Usually, with default estimation, one can leave in random
	terms with 0 variance (although one can always take them out). With
	with these direct likelihood methods, one MUST take them out after
	the initial run.;
*---Switching to NOBOUND does not help here (still blows up).;
	

*---Reasonable results when taking out blk. That is,just use blk*var;
proc glimmix data=pooled method=LAPLACE; *<--this is a correct model;
	title2 'Conditional binomial distribution,  random BLK*TRT (no BLK)';
	title3 'LAPLACE estimation method (approximate the likelihood)';
	class trt blk;
	model y/n = trt / dist=bin link=logit;
	random   blk*trt;	*<--removed the block main effect;
	lsmeans trt / diff lines ilink cl plot=mean(CL ilink) ;
run;


*---Reasonable results when taking out blk. That is,just use blk*var;
proc glimmix data=pooled method=LAPLACE; *<--this is a correct model;
	title2 'Conditional binomial distribution,  random BLK*TRT (no BLK)';
	title3 'LAPLACE estimation method (approximate the likelihood)';
	class trt blk;
	model y/n = trt / dist=bin link=logit;
	random   blk*trt;	*<--removed the block main effect;
	covtest 0 / cl(type=profile);  *<--Conf.Int for variance and test of Ho: variance=0;
	lsmeans trt / diff lines ilink cl plot=mean(CL ilink) ;
run;


proc glimmix data=pooled method=quad;
	title2 'Conditional binomial distribution,  random BLK*TRT (no BLK)';
	title3 'QUADRATURE estimation method (approximate the likelihood), must use subject=';
	class trt blk;
	model y/n = trt / dist=bin link=logit;
	random   trt / sub=blk;	*<--removed the block main effect, must use subject= syntax;
	lsmeans trt / diff lines ilink cl plot=mean(CL) ;
run;

*^--Negative variances (NOBOUND) would also give meaningless results
	with Laplace or quadrature (be careful!).;

*---In summary, Laplace or quadrature are only for true GLMMs with
	positive variances for the random effects. It cannot be used
	for quasi-likelihood (hybrid approaches).;

*---Using glmer {lme4} in R, once can fit GLMMs using quadrature or Laplace (approximating
	the likelihood). As with SAS, one must remove any 0-variance
	random-effect terms before using the results. ;


/*
					GEE (NOT COVERED IN WORKSHOP):
There is another approach, known as Generalized Estimating Equations (GEE).
It became very popular before good GLMM software was available. It is
very popular in medical sciences. We utilize the compound symmetry (CS)
specification:
	random trt / sub=blk type=CS
For true random effects, this structure is the same as 
	random int trt / sub=blk 
if all the variance terms are positive. 
For GEE, we consider heterogeneity to be a "residual" (phi) effect, 
to scale everything for the over-dispersion: 
	random trt / sub=blk type=cs RESIDUAL
It is strictly a fixed-effect 
model analysis, with an adjustment for variances. That is, it is not 
a GLMM (with random effects). It is also a marginal model, 
where the p estimates are across all blocks, NOT for the p at the unit 
level (plot).

Stroup describes in 2013 Agronomy Journal article. Not covered here.
Not easy to generalized to all possible experimental layouts.
*/
 

proc glimmix data=pooled ;
	title2 'Generalized Estimating Equations (GEE)';
	title3 '(not covered in workshop)';
	class trt blk;
	model y/n = trt /  dist=bin link=logit;
	random  trt / sub=blk type=CS residual; 	*<--note the residual option for GEE;
	lsmeans trt / diff lines ilink cl plot=mean(CL ilink) ;
run;
*^--magnitude of "random" variances are very different because
	these are no longer on the scale of the logits, but on scale of data.;



