

/*
			CASE STUDY 2: SPLIT PLOT WITH BLOCKING

Effects of wheat cultivar and fungicide timing (relative to time of 
anthesis) on the incidence of Fusarium head blight of wheat. 

var:		Variety (cultivar), whole-plot factor (i = 1, 2)
trt:		Fungicide treatment, sub-plot factor (j = 1, ..., 5)
			1: control, 2: spray at Anthesis (A), 3: spray at A+2,
			4: spray at A+4; 5: spray at A+6 days after anthesis
blk:		Block (k = 1, 2, 3)
diseased:	Number of diseased spikes (heads): Y
n:			Number of observed spikes for symptoms
incidence:	diseased/n (a proportion)
angular:	arcsine(square-root(incidence))

This is part of a published dataset in 
D'Angelo et al. (2014 Plant Disease 98: 1387-1397).
The full data set also has a third factor, inoculum density, which is a 
sub-sub plot factor. This subset of the observations includes only 
the third level of inoculum density. Thus, the design is a split plot
with blocks. Most of the paper dealt with FHB disease index (basically,
severity of wheat spikes). Here we are looking only at incidence.

If using SAS UE, take out * before INFILE.

Program provided for educational purposes. There are no warranties.
			--  Larry Madden and Alissa Kriss

*/


data sp;
*infile datalines dlm='09'x;	*<--use this line only with SAS UE (for embedded tabs);
input blk var $ trt n diseased;   
incidence = Diseased/n;
angular = arsin(sqrt(incidence));
datalines;
1	Hopewell	4	70	7
1	Hopewell	1	70	27
1	Malabar	5	70	9
1	Malabar	4	70	11
2	Malabar	5	100	4
2	Hopewell	5	100	15
3	Malabar	5	100	9
3	Malabar	1	100	13
3	Hopewell	2	100	11
1	Hopewell	5	70	10
2	Malabar	2	100	10
2	Malabar	4	100	9
2	Hopewell	2	100	9
3	Hopewell	5	100	15
3	Hopewell	4	100	15
1	Hopewell	3	70	13
1	Malabar	2	70	7
2	Malabar	1	100	6
2	Malabar	3	100	7
2	Hopewell	1	100	17
2	Hopewell	3	100	8
3	Malabar	3	100	10
1	Hopewell	2	70	5
1	Malabar	1	70	13
1	Malabar	3	70	3
2	Hopewell	4	100	9
3	Malabar	2	100	7
3	Malabar	4	100	12
3	Hopewell	1	100	19
3	Hopewell	3	100	22
;
run;

title 'Case study 2: Fusarium head blight: whole-plot=variety, sub-plot=fungicide treatment';
proc print data=sp;run;

*---First, a LMM (normal) for transformation. Note that one needs
	to specify block and block*treatment, the latter representing
	the whole-plot random-effect term (combination of blk and trt
	uniquely identify the whole plot).;

ods graphics on / width=5in;
proc sgpanel data=sp;
panelby var;
scatter x=trt y=incidence / markerattrs=(color=blue symbol=circlefilled size=7pt);
colaxis  label="Treatment" type=discrete;
rowaxis label="Disease incidence";
run;


proc glimmix data=sp;
title2 'split plot, LMM (normal distribution for angular transformation)';
class blk var trt ;
model angular = var trt var*trt; *--a normal distribution here (default);
random blk blk*var;	
lsmeans trt / lines plots=mean(cl connect);
run;

*---Note that model statement can be written as: var|trt as short-hand for
	var trt var*trt;



*---First do the naive analysis, borrowing directly from LMM (take the normal-
	based model and substitute binomial for normal. We thus ignore any 
	unique random contribution of the basic experimental unit (sub-plot, which is
	the ijk combination here).;

proc glimmix data=sp;
title2 'split plot, naive GLMM (bin., logit)';
title3 '(not a good model unless there is no effect of exper. unit [sub-plot])';
class blk var trt ;
model diseased/n = var|trt / dist=binomial link=logit;
random blk blk*var;	*<--directly borrowing from LMM, ignoring possible unit-level effects;
lsmeans trt / lines cl plots=mean(cl connect);
run;

  
*---Now we fit a conditional GLMM, explicitly adding in the term for the
	random effect of the basic experimental unit (the sub-plot).;

proc glimmix data=sp;
title2 'split plot, full conditional GLMM (bin., logit), with unit-level random effect';
class blk var trt ;
model diseased/n = var|trt / dist=binomial link=logit;
random blk blk*var blk*var*trt;	*<--unit-level (plot) effects added;
lsmeans trt / lines cl plots=mean(cl connect);
run;
*^--Note that the blk*var random effect is 0. One _could_ remove and re-fit.
	If one used Laplace or Quadrature estimation, one would HAVE to remove
	it. Can be kept here with default model fitting (method=rspl).;


*---Now get inverse link estimates of p (conditional probability of disease);
proc glimmix data=sp;
title2 'split plot, full conditional GLMM (bin., logit), with unit-level random effect';
class blk var trt ;
model diseased/n = var|trt / dist=binomial link=logit;
random blk blk*var blk*var*trt;	*<--unit-level (plot) effects added;
lsmeans trt / ILINK cl lines plots=mean(cl connect ILINK) ;
run;


*---Get contrast of trt 2 versus average of trt 3, 4, and 5. One needs another
	workshop to learn about CONTRAST, ESTIMATE, and LSMESTIMATE statements.;
proc glimmix data=sp;
title2 'split plot, full conditional GLMM (bin., logit), with unit-level random effect';
title3 'demo of the use of LSMESTIMATE statement for customized contrasts';
class blk var trt ;
model diseased/n = var|trt / dist=binomial link=logit;
random blk blk*var blk*var*trt;	*<--unit-level (plot) effects added;
lsmeans trt / lines cl plots=mean(cl connect);
lsmestimate trt '2 vs (3,4,5)' 0 3 -1 -1 -1 / divisor=3;	*<--exampled contrast;
run;


*---demonstration of slices and graphs (if interaction is significant);
proc glimmix data=sp;
title2 'split plot, full conditional GLMM (bin., logit), with unit-level random effect';
class blk var trt ;
model diseased/n = var|trt / dist=binomial link=logit;
random blk blk*var blk*var*trt;	*<--unit-level (plot) effects added;
lsmeans trt / ILINK cl plots=mean(cl connect ILINK) ;
lsmeans var*trt / slice= var ILINK cl plots=mean(sliceby=var cl connect ILINK) ;
run;


*---Now do the alternative analysis based on quasi-likelihood. That is, allow
	for overdispersion (extra-binomial variation) due to the random effect
	of the sub-plots. One no longer has a true distribution or true
	likelihood. One starts with naive GLMM  (see above), and adds a
	residual scale term.;

proc glimmix data=sp;
title2 'split plot, GLMM QUASI-LIKELIHOOD (bin., logit) for overdispersion';
class blk var trt ;
model diseased/n = var|trt / dist=binomial link=logit;
random blk blk*var;	*<-- no effect of blk*var*trt here;
random _residual_; *<--over-dispersion scale parameter (instead of plot rand. effect);
lsmeans trt / lines plots=mean(cl connect);
run;
  
*^--Be careful, IF one used random _residual_ AND the blk*var*trt
	random term, the results would be meaningless (and wrong!). 
	Over parameterized. Either add the unit-level random effect
	(here: blk*var*trt), or add a _residual_ scale term, not both.;




*******************************************************************************;
**************** Try fitting with Laplace or quadrature ***********************;
*******************************************************************************;

proc glimmix data=sp method=laplace;
title2 'LAPLACE fit: split plot, naive GLMM (bin., logit)';
class blk var trt ;
model diseased/n = var|trt / dist=binomial link=logit;
random blk blk*var;	*<--directly borrowing from LMM, ignoring possible unit-level effects;
lsmeans trt / lines cl plots=mean(cl);
run;
*^--The estimated variance of 0 CORRUPTS all the results. One MUST remove
	the offending random-effects term (in this example, blk*var 
	[whole-plot variance]) to use the Laplace/quadrature methods.;


proc glimmix data=sp method=laplace;
title2 'LAPLACE fit: split plot, conditional GLMM (bin., logit), with unit-level effect';
class blk var trt ;
model diseased/n = var|trt / dist=binomial link=logit;
random blk blk*var blk*var*trt;	*<--unit-level (plot) effects added;
lsmeans trt / lines plots=mean(cl);
run;
*^--The estimated variance of 0 corrupts all the results. One must remove
	the offending random-effects term (blk*var in this example
	[whole-plot variance]) to use the Laplace/quadrature methods.;



*---Now take out the 0 variance term and proceed. Note that some results
	are different for Laplace versus linearization (default) model
	fitting (variety may become significant). Variance estimates here
	are generally lower than with default estimation. WIth the small
	number of blocks/reps (3), estimation method can have a bigger
	influence.;

proc glimmix data=sp method=laplace;
title2 'LAPLACE fit: split plot, naive GLMM (bin., logit)';
title3 '(took out blk*var [whole plot rand term] because variance was 0)';
class blk var trt ;
model diseased/n = var|trt / dist=binomial link=logit;
random blk ;	*<--directly borrowing from LMM, removed 0 variance term;
lsmeans trt / lines plots=mean(cl);
lsmestimate trt '2 vs (3,4,5)' 0 3 -1 -1 -1 / divisor=3;
run;


proc glimmix data=sp method=laplace;
title2 'LAPLACE fit: split plot, conditional GLMM (bin., logit), with unit-level effect';
title3 '(took out blk*var [whole plot rand term] because variance was 0)';
class blk var trt ;
model diseased/n = var|trt / dist=binomial link=logit;
random blk   blk*var*trt;	*<--unit-level (plot) effects added, removed 0 variance term;
lsmeans trt / lines plots=mean(cl);
run;
*^--Note that variety (var) is now also significant! So, the conclusions CAN 
	depend on the model-fitting method (with default RSPL method, var was
	not significant). A reason for the difference: the blk*var*trt variance
	estimate is very small using Laplace estimation (much smaller than with
	the default). Thus, there is little evidence of overdispersion here.
	Caution: this is all  model and data dependent (sometimes the results will
	be very close, and sometimes the difference can be in the opposite direction.;



/*	
With Laplace (or quadrature), one can do things that are not possible with
the default RSPL linearization (pseudo-likelihood) method. For instance, one
can determine goodness of fit of the different models (with different random
effects, always making sure that any zero-variance terms are omitted). With
Laplace, one can compare the AIC or AICC statistics for different models to
see if one fits best.  

Final note: One _cannot_ fit the quasi-likelihood model using 
	Laplace (or quadrature). Only true likelihoods are allowed.;


proc glimmix data=sp method=laplace;
title2 'LAPLACE fit: split plot, conditional GLMM (bin., logit), with unit-level effect';
title3 '(took out blk*var [whole plot rand term] because variance was 0)';
class blk var trt ;
model diseased/n = var|trt / dist=binomial link=logit;
random blk   blk*var*trt;	*<--unit-level (plot) effects added, removed 0 variance term;
lsmeans trt / lines plots=mean(cl);
COVTEST . 0 / cl(type=profile); *<--test for last variance=0;
run;
*/






