
/*
		Case Study #4
Effect of all all zeros in treatment 1 on GLMM fit.

Randomized complete block design (RCBD)

blk:	Block (j=1,2,3,4)
trt:	Treatment (i=1,2,3)
y:		Number diseased (or number with trait)
n:		Number of observations (e.g., leaves) in each plot
		(combination of block and treatment -- ij combination)
		=20 in all cases here.

Below we explore several AD HOC adjustments to give sensible results.
No clear best approach. Alternative: remove the all-zero (or 
all-n) treatment. Not done here. 

Program provided for educational purposes. There are no warranties.
			-- Larry Madden and Alissa Kriss (2016)
*/


title 'Case study #4: RCBD, binomial, all zero for treatment 1';
data zero;
input blk trt y;
n = 20;
datalines;
1 1 0
1 2 2
1 3 4
2 1 0
2 2 6
2 3 8
3 1 0
3 2 0
3 3 2
4 1 0
4 2 7
4 3 7
;
run;
proc sort data=zero;
by trt;
run;
proc print data=zero;
run;

proc glimmix data=zero method=rspl;	*<--default estimation, will not converge;
title2 'Conditional binomial, logit. See impact of all zero for treat 1';
title3 '(does not converge with default linearization fitting method)';
class blk trt;
model y/n = trt /  link=logit s ;
random blk blk*trt;
lsmeans trt / diff cl;
nloptions maxiter=1000 maxfunc=1000 gconv=1e-6;	*<--try to help with convergence;
parms (.3) (.5);	*<--initial guess of variances (try to help with convergence);
run;

*---One _can_ get convergence using method=LAPLACE (see code at end of file). 
	But, one obtains extremely large negative logit mean, and extremely
	large SE for the logit mean (not meaningful). Remember, if one uses one of the
	likelihood approximation methods (laplace, quadrature), one needs to
	take out zero variance random terms _before_ looking at any other results.;

/* 
Note that the smallest possible nonzero value of y is 1 (with n being the max).
A typical correction involves adding 0.5 (half of the smallest real
nonzero value) to y, and proceeding with the analysis. There are
several approaches to this. In the first situation, we modify
just one observation of the treatment with all zeros. This ensures
that the model fitting is possible. 

Then we modify all the zeros, and then we modify all the observations.
If there are cases of all y values equal to n, then we modify these also.

Basic idea: convert y/n to yprime/nprime. One equation:
	yprime = (y+c)
	nprime  = (n+2c)
At 50%, yprime/nprime = y/n = 0.50 (unbiased at 1/2). 

The standard approach is to use c=0.5. So,
	yprime=y+0.5
	nprime=n+1

*/



*---Modified the data, slightly, for one rep of trt 1....
	This is one way to at least get convergence and results. 
	May not be best.;

data zero; set zero;	*<--we changed the first blk of trt 1 to 0.5;
y2 = y;	*<--new y;
n2 = n;	*<--new n;
if ((trt eq 1) and (blk eq 1)) then do;
	y2 = 0.5;	n2=n+1;	end;
run;
proc print data=zero; run;


proc glimmix data=zero ;
	title2 'Conditional binomial distribution, logit';
	title3 'Modified trt1,blk1 only to y+1/2, n+1';
	class trt blk;
	model y2/n2 = trt /  dist=bin link=logit;
	random   blk blk*trt; *<--could take out blk*trt here because of 0 variance;
	lsmeans trt / diff lines cl plot=mean(CL) ;
	nloptions maxiter=1000 maxfunc=1000 gconv=1e-7;	*<--try to help with convergence;
run;
*^--This now runs, and could produce reasonable results. However, this is
	not really adequate with this dataset (very large SE for treatment 1 mean).;




*---Now modified the data, slightly, for ALL treatments and reps....;

data zero; set zero;
y3 = y+0.5;
n3 = n+1;	*<--if you add c to y, you must add 2c to n;
run;
proc print data=zero; run;


*---Must use y3/n3 (not y/n or y2/n2) in model statement.
	This is based on modifying all observations.;
proc glimmix data=zero ;
	title2 'Conditional binomial distribution, logit';
	title3 'Modified (adjusted) all observations, y+1/2, n+1';
	class trt blk;
	model y3/n3 = trt /  dist=bin link=logit;
	random   blk blk*trt; 
	lsmeans trt / diff lines cl plot=mean(CL) ;
	nloptions maxiter=1000 maxfunc=1000 gconv=1e-7;	*<--try to help with convergence;
run;



*---Now modify all the y=0s, and all the y=ns, only (leave the rest alone) ;
data zero; set zero;
y4 = y;
n4=n;
if ((y eq 0) or (y eq n)) then do;
	y4 = y+0.5;
	n4 = n+1;
	end;				*<--just changed the 0s and ns;
run;
proc print data=zero; run;


*---Must use y4/n4 (not y/n or y2/n2) in model statement.
	This is for a modification of 0 and n only.;
proc glimmix data=zero ;
	title2 'Conditional binomial distribution, logit';
	title3 'Modified (adjusted) all 0 and n observations to y+1/2 n+1';
	class trt blk;
	model y4/n4 = trt /  dist=bin link=logit;
	random   blk blk*trt; 
	lsmeans trt / diff lines cl plot=mean(CL) ;
	nloptions maxiter=1000 maxfunc=1000 gconv=1e-7;	*<--try to help with convergence;
run;


*---Repeat above, but add ILINK.;
proc glimmix data=zero ;
	title2 'Conditional binomial distribution, logit';
	title3 'Modified (adjusted) all 0 and n observations to y+1/2 n+1';
	class trt blk;
	model y4/n4 = trt /  dist=bin link=logit;
	random   blk blk*trt; 
	lsmeans trt / diff lines ILINK cl plot=mean(CL ILINK) ;
	nloptions maxiter=1000 maxfunc=1000 gconv=1e-7;	*<--try to help with convergence;
run;

*---Another approach to to couple the above with a different link function.
	The probit link is generally less sensitive to the "all zero problem".
	One still needs one of the data adjustments (see above).;


*---Try probit with the case where all zeros and ns were adjusted.;
*---Must use y4/n4 (not y/n or y2/n2) in model statement;
proc glimmix data=zero ;
	title2 'Conditional binomial distribution, PROBIT link (not logit)';
	title3 'Modified (adjusted) all 0 and n observations';
	class trt blk;
	model y4/n4 = trt /  dist=bin link=probit;
	random   blk blk*trt; 
	lsmeans trt / diff lines cl plot=mean(CL) ;
	nloptions maxiter=1000 maxfunc=1000 gconv=1e-7;	*<--try to help with convergence;
run;
*^--Note that the SE for treatment 1 is less different from the SE for the other
	two treatments. That is, there is less of an artifact. Probit link might
	be less sensitive to this problem (???).;


**********************************************************************************;
**********************************************************************************;
****************** Fit with direct likelihood method *****************************;
**********************************************************************************;
**********************************************************************************;

*---If we cover this case study before covering Laplace method, then we will
	not cover the material below these comments.;

*---One can do this with method=LAPLACE. However, one must first _remove_
	any random effects with 0 variance estimate. Without doing this, one
	cannot see the effects of all-zeros on the results.
	That is, after the first run with convergence, take out the blk*trt
	random effect because it has a zero variance (in this example).;
*---With Laplace, one can get convergence without any data modifications,
	but results for trt 1 (and maybe the rest) are meaningless.;


proc glimmix data=zero method=LAPLACE;	*<--change estimation method;
title2 'See the impact of all zero for treatment 1';
title3 '(converges with Laplace likehood approximation method)';
title4 '(but one of the random effects is 0 [results are thus meaningless], need to take out)';
class blk trt;
model y/n = trt /  link=logit  ;
random blk blk*trt;
lsmeans trt / diff cl plot=mean(cl);
run;
*^--converges with Laplace. But blk*trt variance is 0. First rule for 
	LAPLACE, take out 0 variance terms. Then re-do. So, take out blk*trt.;


*---Took out blk*trt random effect. MUST do this before looking at anything
	else for the Laplace or Quadrature model-fitting methods.;

proc glimmix data=zero method=LAPLACE;	*<--change estimation method;
title2 'See the greatly inflated trt 1 effect and other distorted statistics';
title3 '(converges with Laplace likehood approximation method)';
class blk trt;
model y/n = trt /  link=logit s ;
random blk ;								*<--took out blk*trt;
lsmeans trt / diff cl plot=mean(cl);
run;
*^--poor results because of all-zeros for trt 1;



proc glimmix data=zero method=LAPLACE;
	title2 'Conditional binomial distribution, logit';
	title3 'LAPLACE: Modified trt1,blk1 only to y+1/2 n+1';
	class trt blk;
	model y2/n2 = trt /  dist=bin link=logit;
	random   blk ; 
	lsmeans trt / diff lines cl plot=mean(CL) ;
run;


proc glimmix data=zero method=LAPLACE;
	title2 'Conditional binomial distribution, logit';
	title3 'LAPLACE: Modified (adjusted) _all_ observations to y+1/2 n+1';
	class trt blk;
	model y3/n3 = trt /  dist=bin link=logit;
	random   blk; 
	lsmeans trt / diff lines cl plot=mean(CL) ;
run;


proc glimmix data=zero method=LAPLACE;
	title2 'Conditional binomial distribution, logit';
	title3 'LAPLACE: Modified (adjusted) all 0 and n observations to y+1/2 and n+1';
	class trt blk;
	model y4/n4 = trt /  dist=bin link=logit;
	random   blk; 
	lsmeans trt / diff lines cl plot=mean(CL ) ;
run;




